//************************************************************************//
//*************CASPER....na bazie projektu ZŁOMEK************
// Projekt otwarty http://sp-hm.pl/thread-2164.html
// Kompilowany IDE Arduino ver 1.06
//************************************************************************//

//podłączamy bibliotekę syntezera
#include <AH_AD9850.h>
//podłączamy bibliotekę enkodera
#include <RotaryEncoder.h>;
//podłączamy bibliotekę do obsługi wyświetlacza
#include <LCD5110_Graph.h>

//inicjalizujemy komunikację z syntezerem
//syntezer   - arduino
//CLK        - PIN 9
//FQUP       - PIN 10
//BitData    - PIN 11
//RESET      - PIN 12
AH_AD9850 AD9850(9, 10, 11, 12);

// inicjalizujemy wyświetlacz
/*    SCLK      - Pin 7
      MOSI(DIN) - Pin 6
      DC        - Pin 5
      RST       - Pin 3
      CE        - Pin 4
*/
LCD5110 myGLCD(7,6,5,3,4);


extern uint8_t TinyFont[];  //czcionka z biblioteki.....dodałem małe fonty
extern uint8_t SmallFont[]; //czcionka z biblioteki
extern uint8_t MediumNumbers[];//czcionka z biblioteki

//inicjalizujemy enkoder
//A2 - w lewo
//A3 - w prawo
//nalezy pamiętać o kondensatorach (100nF) pomiędzy liniami encodera a masą
RotaryEncoder encoder(A2,A3,5,6,1000);

//*****************************************************************************************************************************
//zmienne do modyfikacji każdy ustawia to co potrzebuje
//.....tu są ustawienia moje.......
const int step_input = A0;                   //wejście do podłączenia przełącznika zmiany kroku
const int s_metr_port = A1;                  //wejście dla s-metra
const int rit_swich_input = 2;               //wejście do uruchamiania funkcji RIT
const int ptt_input = 0;                     //wejście PTT procesor musi wiedzieć czy nadajemy czy odbieramy by zrealizować RIT-a
//const int bandPin_sw_a = 1;                  //switch - przelaczanie BAND_ góra
//const int bandPin_sw_d = 2;                  //switch - przelaczanie BAND_ dół
const int bandPin_out = 13;                  // wyjscie przelaczania BAND
const int kontrast = 70;                     //kontrast wyświetlacza
const int pulses_for_groove = 2;             //ilość impulsów na ząbek enkodera zmienić w zależności od posiadanego egzemplarza
const long low_frequency_limit = 3490000;    //dolny limit częstotliwości
const long switch_band_frequency = 5000000;  // freq przelaczania pasm REV.13.07.2020
const long high_frequency_limit = 7210000;   //górny limit częstotliwości
const long start_frequency = 3699000;        //częstotliwość startowa syntezy_vfoA
const long posrednia = 9012300;              //częstotliwość pośredniej, każdy dobiera swoją w zależności od konstrukcji radia
const int tryb_pracy = 0;                    //tryby pracy: 0-pośrednia, 1-generator, 2-lub wyżej, mnożnik razy 2 lub więcej
long step_value = 1000;                      //domyślny krok syntezy
const long s_metr_update_interval = 100;     //interwał odświeżania s-metra w msec
const long rit_range = 2000;                 //zakres pracy RIT +/- podana wartość, domyślnie 2000Hz max 9999Hz jeśli dasz wiecej posypie się wyświetlanie
const long rit_step = 50;                    //krok działania RIT-a domyślnie 50Hz
//*****************************************************************************************************************************

//zmienne wewnętrzne pomocnicze,
//jeśli nie trzeba proszę nie modyfikować
char buffor[] = "              ";            //zmienna pomocnicza do wyrzucania danych na lcd
long frequency = start_frequency;            //zmienna dla częstotliwości, wstawiamy tam częstotliwość od której startujemy
int enc_sum = 0;                             //zmienna pomocnicza do liczenia impulsów z enkodera
long s_metr_update_time = 0;                 //zmienna pomocnicza do przechowywania czasu następnego uruchomienia s-metra
long frequency_to_dds = 0;                   //zmienna pomocnicza przechowuje częstotliwość którą wysyłam do DDS-a
int rit_state = 0;                           //stan RIT-a 0-rit off, 1-rit on, 2-rit on enkoder odchyłkę rit
boolean ptt_on = false;                      //stan przycisku PTT
boolean last_ptt_state = false;              //poprzedni stan PTT potrzebne do wykrywania zmianu stanu PTT
long rit_poprawka = 0;                       //domyślna wartość poprawki
//*****************************************************************************************************************************

//FUNKCJE
//funkcja do obsługi wyświetlania zmiany częstotliwości
void show_frequency(){
  if(rit_state != 1){                              //Jeśli Enkoder pracuje jako VFO zaoszczędzimy trochę czasu procesora jesli enkoder pracuje jako RIT
    long f_prefix = frequency/1000;                //pierwsza część częstotliwości dużymi literkami
    long f_sufix = frequency%1000;                 //obliczamy resztę z częstotliwości
    sprintf(buffor,"%05lu",f_prefix);              //konwersja danych do wyświetlenia (ładujemy częstotliwość do stringa i na ekran)
    myGLCD.setFont(MediumNumbers);                 //ustawiamy czcionkę dla dużych cyfr
    myGLCD.print(buffor,1,13);                     //wyświetlamy duże cyfry na lcd
    sprintf(buffor,".%03lu",f_sufix);              //konwersja danych do wyświetlenia (ładujemy częstotliwość do stringa i na ekran)
    myGLCD.setFont(SmallFont);                     //ustawiamy małą czcionkę
    myGLCD.print(buffor,60,22);                    //wyświetlamy małe cyfry na lcd
  }
  if(rit_state == 1){                              //jeśli RIT jest włączony i enkoder pracuje jako RIT wyświetlamy zmiany częstotliwości RIT-a
    myGLCD.setFont(TinyFont);                      //ustawiamy małą czcionkę
    sprintf(buffor,"%05lu",abs(rit_poprawka));     //przygotowujemy bufor z zawartością aktualnej wartości RIT
    myGLCD.print(buffor,CENTER,2);                 //drukowanie na lcd
    if(rit_poprawka < 0){                          //obsługa znaku poprawki RIT jeśli mniejsza niż 0
      myGLCD.print("-",28,2);                      //drukujemy minus
    }else if(rit_poprawka > 0){                    //jeśli większa niż zero to
      myGLCD.print("+",28,2);                      //drukujemy plus
    }
    else{
      myGLCD.print("0",28,2);                      //jeśli poprawka zerowa wrzucam zero zamiast plusa czy minusa
    }
  }
  myGLCD.update();                                 //wysyłamy dane do bufora wyświetlacza
}

//funkcja do wyświetlania aktualnego kroku syntezera za pomocą podkreślenie odpowiedniej cyfry
void show_step(){
       myGLCD.clrLine(0,31,83,31);          //czyszczę cała linię gdzie wyświetlam podkreślenie
       myGLCD.clrLine(0,32,83,32);          //czyszczę druga linię tak samo podkreśliniki są grube na dwa piksele
  switch(step_value){                       //przełącznik reaguje na bieżącą wartość kroku syntezy
     case 50:
        myGLCD.drawLine(69, 31, 82, 31);    //pokreślam 100Hz
        myGLCD.drawLine(69, 32, 82, 32);    //pokreślam 100Hz
     break;
    case 1000:
        myGLCD.drawLine(51, 31, 59, 31);    //pokreślam 1kHz
        myGLCD.drawLine(51, 32, 59, 32);    //pokreślam 1kHz
    break;
    case 10000:
        myGLCD.drawLine(39, 31, 47, 31);    //pokreślam 10kHz
        myGLCD.drawLine(39, 32, 47, 32);    //pokreślam 10kHz
    break;
    case 100000:
        myGLCD.drawLine(27, 31, 35, 31);    //pokreślam 100kHz
        myGLCD.drawLine(27, 32, 35, 32);    //pokreślam 100kHz
    break;
  }
 myGLCD.update();  //jak już ustaliliśmy co rysujemy to wysyłamy to do LCD
}

//funkcja ustawiająca częstotliwość DDS-a
void set_frequency(int plus_or_minus){
  if(rit_state == 2 || rit_state == 0){                                            //jeśli nie obsługuję RIT-a to manipuluje częstotliwością
    if(plus_or_minus == 1){                                                        //jeśli na plus to dodajemy
      frequency = frequency + step_value;                                          //częstotliwość = częstotliwość + krok
    }
    else if(plus_or_minus == -1){                                                  //jeśli na minus to odejmujemy
      frequency = frequency - step_value;                                          //częstotliwość = częstotliwość - krok
    }
  }
  if(rit_state == 1){                                                              //jesli obsługuję rita
    if(plus_or_minus == 1){                                                        //jeśli na plus to dodajemy
      rit_poprawka = rit_poprawka + rit_step;                                      //częstotliwość poprawki zwiększam o krok poprawki
    }
    else if(plus_or_minus == -1){                                                  //jeśli na minus to odejmujemy
      rit_poprawka = rit_poprawka - rit_step;                                      //częstotliwość poprawki zmniejszam o krok poprawki
    }
    rit_poprawka = constrain(rit_poprawka,-rit_range,rit_range);                     //limitujemy poprawkę RIT do wartości z konfiguracji
  }
  int poprawka = 0;                                                                //lokalna zmienna pomocnicza
  if(rit_state != 0 && ptt_on == false){                                           //jeśli jesteśmy w trybie włączonego RIT-a
    poprawka = rit_poprawka;                                                       //lokalna zmienna pomocnicza przyjmuje wartość RIT by można to było dodać do czestotliwości
  }
  frequency = constrain(frequency,low_frequency_limit,high_frequency_limit);       //limitowanie zmiennej częstotliwości tej na wyświetlaczu
  if(tryb_pracy == 0){                                                             //zmiana trybu pracy syntezy 0 - pośrednia
    frequency_to_dds = abs(posrednia + frequency + poprawka);                      //a tutaj obliczam częstotliwość wynikową dla pracy w trybie pośredniej + ew.poprawka z RIT
  }else{                                                                           //tryby pracy 1 - mnożnik * 1 generator lub 2 i więcej mnożnik
    frequency_to_dds = (frequency + poprawka) * tryb_pracy;                        //mnożymy częstotliwość przez tryb pracy no i pamiętamy o poprawce
  }

  // automatyczne przelaczanie pasma REV.13.07.2020
  digitalWrite(bandPin_out, (frequency<switch_band_frequency)? LOW : HIGH);

  AD9850.set_frequency(frequency_to_dds);                                          //ustawiam syntezę na odpowiedniej częstotliwości
  Serial.println(frequency_to_dds);                                              //debugowanie
}
//set_frequency

//wskaźnik s-metra by nie przeszkadzał w pracy enkodera zrobiony jest na pseudo współdzieleniu czasu.
//każdorazowe wykonanie tej funkcji przestawia czas następnego jej wykonania o czas podany w konfiguracji domyślnie 100msec
void show_smetr(){
  if(millis() >= s_metr_update_time){                              //sprawdzam czy już jest czas na wykonanie funkcji
     myGLCD.clrLine(1, 45, 83, 45);                                //czyścimy stare wskazanie s-metra linia pierwsza
     myGLCD.clrLine(1, 46, 83, 46);                                //czyścimy stare wskazanie s-metra linia druga
     //int s_value = analogRead(A5);                                 //czytamy wartość z wejścia gdzie jest podpięty sygnał s-metra
     int s_value = analogRead(A1);

//     int s_position = map(s_value,0,1023,1,83);                    //przeskalowuję zakres z wejścia analogowego na szerokość wyświetlacza
     //int s_position = (s_value*10)>>7;                           //alternatywny sposób przeskalowania SP9MRN
     const int ADC_VALUE_LIMIT=800;  // REV.05.09.2020 - dobrać limit (max.1023) w zależności od maksymalnego napięcia na wejściu SMETR-a
     if(s_value>ADC_VALUE_LIMIT) s_value=ADC_VALUE_LIMIT;
     int s_position = map(s_value,0,ADC_VALUE_LIMIT,1,83);         //przeskalowuję zakres z wejścia analogowego na szerokość wyświetlacza

     myGLCD.drawLine(1, 45, s_position, 45);                       //rysuję nową linię wskazania s metra
     myGLCD.drawLine(1, 46, s_position, 46);                       //rysuję nową linię wskazania s metra
     myGLCD.update();                                              //wysyłam dane do bufora wyświetlacza
     s_metr_update_time = millis() + s_metr_update_interval;       //ustawiam czas kolejnego wykonania tej funkcji
  }
}

//funkcja która obsługuje klawisz RIT-a
void rit_swich(){
 myGLCD.setFont(TinyFont);
 switch(rit_state){
  case 1:  //rit aktywny, enkoder steruje wartoscią RIT-a
     myGLCD.drawLine(28, 0, 51, 0);             //pokreślam wartość RIT
     myGLCD.drawLine(28, 8, 51, 8);             //pokreślam wartość RIT
     myGLCD.drawLine(72,0,83,0);                //podkreślam oznaczenie rit
     myGLCD.drawLine(72,8,83,8);                //podkreślam oznaczenie rit
     show_frequency();
  break;
  case 2:  //rit aktywny, enkoder steruje częstotliwością
     myGLCD.clrLine(28, 0, 51, 0);              //anauluję podkreślenie wartości RIT
     myGLCD.clrLine(28, 8, 51, 8);              //anauluję podkreślenie wartości RIT
     myGLCD.drawLine(72,0,83,0);                //podkreślam oznaczenie rit
     myGLCD.drawLine(72,8,83,8);                //podkreślam oznaczenie rit
  break;
  case 0:  //rit nie jest aktywny
     myGLCD.clrLine(28, 0, 51, 0);              //anauluję podkreślenie wartości RIT
     myGLCD.clrLine(28, 8, 51, 8);              //anauluję podkreślenie wartości RIT
     myGLCD.clrLine(72,0,83,0);                 //anuluję oznaczenie rit
     myGLCD.clrLine(72,8,83,8);                 //anuluję oznaczenie rit
     sprintf(buffor,"        ",rit_poprawka);   //czyszczę miejsce po wartości RIT gdy pracujemy bez niego
     myGLCD.print(buffor,CENTER,2);             //przygotowuję dane do wysyłki na LCD
  break;
 }
  myGLCD.update();                              //i wypluwamy to na lcd
  set_frequency(0);                             //ustawiam częstotliwość pracy syntezy na wypadek gdy wartość rit jest różna od zera
}

//sygnalizacja PTT (sygnalizacja to skutek uboczny dla RIT-a musimy wiedzieć czy odbieramy czy nadajemy)
void ptt_switch(){
  myGLCD.setFont(TinyFont);                //ustawiam małą czcionkę
  if(last_ptt_state != ptt_on){            //sprawdzam czy zmienił się stan PTT jeśli tak to robię update na LCD
    if(ptt_on == true){                    //jeśli TX
       myGLCD.print("T", 0,2);             //to zapalamy T do TX
    }else{                                 //jesli RX
       myGLCD.print("R", 0,2);             //zapalamy R do RX
    }
    myGLCD.update();                       //wyrzucam do bufora wyświetlacza
    set_frequency(0);                      //ustawiam częstotliwość (bo może być różna dla TX i RX)
    last_ptt_state = ptt_on;               //uaktualniam poprzedni stan PTT
  }
}

//tutaj rysujemy domyślne elementy ekranu będziemy to robić raz,
//tak by nie przerysowywać całego ekranu podczas pracy syntezy
void show_template(){
  myGLCD.setFont(TinyFont);                           //najmniejsza czcionka
  myGLCD.print("RX", 0,2);                            //Sygnalizacja TX RX będzie tutaj
  myGLCD.print("RIT", 72,2);                          //Sygnalizacja pracy RIT-u tutaj
  myGLCD.print("S1.3.5.7.9.+20.40.60.", CENTER, 38);  //opis dla s-metra
  myGLCD.drawRect(0, 44, 83, 47);                     //rysujemy prostokąt dla s-metra podając koordynaty narożników
  myGLCD.update();                                    //i wypluwamy to na lcd
}

//*****************************************************************************************************************************
//FUNKCJE OBOWIĄZKOWE
//setup funkcja odpalana przy starcie
void setup(){
//  analogReference(DEFAULT);               //konfiguracja napięcia odniesienia
  analogReference(INTERNAL);              //REV.05.09.2020 : ADC reference = 1.1 volts on the ATmega328P
  pinMode(s_metr_port,INPUT);             //ustawiam tryb pracy wejścia s-metra
  pinMode(ptt_input,INPUT_PULLUP);        //ustawiam tryb pracy wejścia PTT
  pinMode(rit_swich_input,INPUT_PULLUP);  //ustawiam tryb pracy wejścia przełącznika RIT
  //pinMode(bandPin_sw_a,INPUT_PULLUP);     //tryb pracy wejścia przełącznika BAND góra
  //pinMode(bandPin_sw_d,INPUT_PULLUP);     //tryb pracy wejścia przełącznika BAND dół
  pinMode(bandPin_out,OUTPUT);            //tryb OUTPUT REV.13.07.2020
  Serial.begin(9600);                     //uruchamiam port szeregowy w celach diagnostycznych
  myGLCD.InitLCD(kontrast);               //odpalamy lcd ustawiamy kontrast
  myGLCD.clrScr();                        //czyścimy ekran z ewentualnych śmieci
  pinMode(step_input,INPUT_PULLUP);       //inicjalizujemy wejście zmiany kroku i podciągamy je do plusa

  set_frequency(0);                       //odpalamy syntezer i ustawiamy częstotliwość startową
//  delay(1000);                            //sekunda opóźnienia
                                           //REV. 05.09.2020 : "ruszenie ennkoderem" przy starcie...
  delay(500);                             //opóźnienie 500ms
  set_frequency(1);                       // "ruszamy encoderem" i ustawiamy częstotliwość
  delay(500);                             //opóźnienie 500ms

  show_frequency();                       //pokazujemy częstotliwość na lcd
  show_step();                            //pokazujemy krok syntezy
  show_template();                        //pokazujemy domyślne stałe elementy LCD
}

void loop(){
  //obsługa PTT to musi być szybkie
  if(digitalRead(ptt_input) == LOW){       //odczytuję wejście PTT jesli jest aktywne
    ptt_on = true;                         //ustawiam zmienna pomocniczą na prawda (flaga)
  }else{                                   //jeśli wejscie nie jest aktywne to
    ptt_on = false;                        //ustawiam zmienną pomocniczą na fałsz (zdejmuję flagę)
  }
  ptt_switch();                            //wywołuję funkcję do obsługi PTT

  //obsługa enkodera
  int enc = encoder.readEncoder();        //czytamy wartość z encodera
  if(enc != 0) {                          //jeśli wartość jest inna niż zero sumujemy
    enc_sum = enc_sum + enc;              //jeden ząbek encodera to +2 lub -2 tutaj to zliczam
    //Serial.println(enc);                //wyrzucamy na port rs-232 wartość licznika enkodera (debugowanie)
    //przesuwam w czasie kolejne wykonanie updejtu s-metra
    //bardzo to pomaga przy szybkim kręceniu enkoderem, nie gubi wtedy kroków
    s_metr_update_time = millis() + s_metr_update_interval;
  }
  //jesli zaliczyliśmy ząbek dodajemy lub odejmujemy do częstotliwości wartość kroku
  if(enc_sum >= pulses_for_groove){
    set_frequency(1);                     //wywołuję funkcje zmiany częstotliwości z parametrem +
    show_frequency();                     //drukuję częstotliwość na wyświetlaczu za pomocą gotowej funkcji
    enc_sum = 0;                          //reset zmiennej zliczającej impulsy enkodera
  }
  if(enc_sum <= -(pulses_for_groove)){
    set_frequency(-1);                    //wywołuję funkcje zmiany częstotliwości z parametrem -
    show_frequency();                     //drukuję częstotliwość na wyświetlaczu za pomocą gotowej funkcji
    enc_sum = 0;                          //reset zmiennej zliczającej impulsy enkodera
  }
  delayMicroseconds(5);                   //małe opóźnienie dla prawidłowego działania enkodera

  //obsługa klawisza zmiany kroku
  if(digitalRead(step_input) == LOW){     //sprawdzanie czy przycisk jest wcisnięty
    delay(50);                            //zwłoka by wyeliminować drgania styków
    if(digitalRead(step_input) == LOW){   //jeśli klawisz nadal jest wcisnięty (czyli nie są to zakłócenia)
      switch(step_value){                 //za pomocą instrukcji swich zmieniamy krok
      case 100000:                        //jeśli krok jest 100kHz ustaw 10kHz
        step_value = 10000;
        break;
      case 10000:                         //jeśli krok jest 10kHz ustaw 1kHz
        step_value = 1000;
        break;
      case 1000:                          //jeśli krok jest 1kHz ustaw 50Hz
        step_value = 50;
        break;
      case 50:
        step_value = 100000;
      break;
      }
     show_step();                          //pokazuję zmianę kroku na lcd
     delay(200);                           //zwłoka po zmianie kroku 200msec
    }
  }

  //obsługa klawisza włączenia funkcji RIT
  if(digitalRead(rit_swich_input) == LOW){    //jeśli klawisz wciśnięty
   delay(50);                                 //zwłoka by wyeliminować drgania styków
    if(digitalRead(rit_swich_input) == LOW){  //jeśli nadal wciśnięty (eliminuję drgania styku)
       switch(rit_state){                     //przełącznik trybu pracy z RIT
        case 1:                               //jesli tryb jest 1 (enkoder pracuje jako rit wartość RIT dodaję do częstotliwości)
         rit_state = 2;                       //ustaw tryb 2
        break;
       case 2:                                //jeśli tryb jest 2 (enkoder pracuje jako enkoder wartość RIT dodaję do częstotliwości)
        rit_state = 0;                        //ustaw tryb 0
       break;
       case 0:                                //jeśli tryb jest 0 (enkoder pracuje jako enkoder wartość RIT wyłączony)
        rit_state = 1;                        //ustaw tryb 2
       break;
       }
       rit_swich();                           //odpalam funkcję do obsługi trybu pracy
    delay(200);                               //zwłoka dla prawidłowego działania
    }
  }

/* wylaczone - REV.13.07.2020
  //obsłuda przycisku bandPin
  if(digitalRead(bandPin_sw_a) == LOW)         //sprawdzam czy wciśnięte
  {
    digitalWrite(bandPin_out, HIGH);
    while(digitalRead(bandPin_sw_a) == LOW);
  }
  if(digitalRead(bandPin_sw_d) == LOW)         //sprawdzam czy wciśnięte
  {
    digitalWrite(bandPin_out, LOW);
    while(digitalRead(bandPin_sw_d) == LOW);
  }
*/

  show_smetr();                               //wywołuję funkcję do obsługi s-metra
  //Serial.println(freeRam());                //testowanie ilości dostępnego RAM-u aby zadziałało należy odkomentować funkcję poniżej
}

//KONIEC PROGRAMU
//*****************************************************************************************************************************

//testowanie ilości dostępnego RAMU
//int freeRam () {
// extern int __heap_start, *__brkval;
// int v;
// return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval);
// }

